<?php

namespace Drupal\skipto\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Configure settings for this site.
 */
class SkiptoSettings extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'skipto_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return ['skipto.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $config = $this->config('skipto.settings');

    $form['buttonSettings'] = [
      '#type' => 'details',
      '#title' => $this->t('Button'),
      '#description' => $this->t('The following options are useful for identify where the menu will be in the DOM structure of the page and which elements will be used as the container for the menu button. The options are of type string.'),
      '#open' => TRUE,
      '#tree' => TRUE,
    ];

    $form['buttonSettings']['containerElement'] = [
      '#type' => 'select',
      '#title' => $this->t('Container Element'),
      '#description' => $this->t('What element wraps the SkipTo button.'),
      '#options' => [
        'nav' => $this->t('Nav'),
        'div' => $this->t('Div'),
      ],
      '#default_value' => $config->get('buttonSettings.containerElement'),
    ];

    $form['buttonSettings']['displayOption'] = [
      '#type' => 'select',
      '#title' => $this->t('Display Option'),
      '#description' => $this->t('How to display the SkipTo button.'),
      '#options' => [
        'fixed' => $this->t('fixed'),
        'popup' => $this->t('popup'),
        'popup-border' => $this->t('popup-border'),
        'static' => $this->t('static'),
      ],
      '#default_value' => $config->get('buttonSettings.displayOption'),
    ];

    $form['buttonSettings']['highlightTarget'] = [
      '#type' => 'select',
      '#title' => $this->t('Highlighting targets'),
      '#description' => $this->t('How to highlight SkipTo targets.'),
      '#options' => [
        'disabled' => $this->t('disabled'),
        'instant' => $this->t('instant'),
        'smooth' => $this->t('smooth'),
        'enabled' => $this->t('enabled'),
      ],
      '#default_value' => $config->get('buttonSettings.highlightTarget'),
    ];

    $form['buttonSettings']['positionLeft'] = [
      '#type' => 'number',
      '#title' => $this->t('Example Number Field'),
      '#description' => $this->t('The position of the “Skip To Content” button from left margin.'),
      '#default_value' => $config->get('buttonSettings.positionLeft'),
      '#min' => 0,
      '#max' => 100,
      '#step' => 1,
    ];

    $form['buttonSettings']['fontSize'] = [
      '#type' => 'textfield',
      '#title' => $this->t('CSS font size'),
      '#description' => $this->t('Set the CSS font-size using the configuration object. The default is inheriting the font size from the page.'),
      '#default_value' => $config->get('buttonSettings.fontSize'),
    ];

    $form['buttonSettings']['fontFamily'] = [
      '#type' => 'textfield',
      '#title' => $this->t('CSS font string'),
      '#description' => $this->t('Set the CSS font-family using the configuration object. The default is inheriting the font family from the page.'),
      '#default_value' => $config->get('buttonSettings.fontFamily'),
    ];

    $form['semantics'] = [
      '#type' => 'details',
      '#title' => $this->t('Landmark & Headings'),
      '#description' => $this->t('The following options are useful for identify where the menu will be in the DOM structure of the page and which elements will be used as the container for the menu button. The options are of type string.'),
      '#open' => TRUE,
      '#tree' => TRUE,
    ];

    $form['semantics']['headings'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Headings to aggregate'),
      '#default_value' => $config->get('semantics.headings'),
      '#options' => [
        "h1" => "h1",
        "h2" => "h2",
        "h3" => "h3",
        "h4" => "h4",
        "h5" => "h5",
        "h6" => "h6",
        "main-only" => $this->t("main-only"),
      ],
      '#description' => $this->t('The level of headings to make the menu from (Default is h1, h2).'),
      '#required' => TRUE,
    ];

    $form['semantics']['landmarks'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Landmark roles to aggregate'),
      '#default_value' => $config->get('semantics.landmarks'),
      '#options' => [
        "banner" => $this->t("banner"),
        "complementary" => $this->t("complementary"),
        "contentinfo" => $this->t("contentinfo"),
        "main" => $this->t("main"),
        "navigation" => $this->t("navigation"),
        "region" => $this->t("region"),
        "search" => $this->t("search"),
      ],
      '#description' => $this->t('A space separated list of landmark names.'),
      '#required' => TRUE,
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $config = $this->config('skipto.settings');
    $config
      ->set('buttonSettings.containerElement', $form_state->getValue([
        'buttonSettings',
        'containerElement',
      ]))
      ->set('buttonSettings.displayOption', $form_state->getValue([
        'buttonSettings',
        'displayOption',
      ]))
      ->set('buttonSettings.highlightTarget', $form_state->getValue([
        'buttonSettings',
        'highlightTarget',
      ]))
      ->set('buttonSettings.positionLeft', $form_state->getValue([
        'buttonSettings',
        'positionLeft',
      ]))
      ->set('buttonSettings.fontSize', $form_state->getValue([
        'buttonSettings',
        'fontSize',
      ]))
      ->set('buttonSettings.fontFamily', $form_state->getValue([
        'buttonSettings',
        'fontFamily',
      ]))
      ->set('semantics.headings', $this->cleanupMap($form_state->getValue([
        'semantics',
        'headings',
      ])))
      ->set('semantics.landmarks', $this->cleanupMap($form_state->getValue([
        'semantics',
        'landmarks',
      ])))
      ->save();
    parent::submitForm($form, $form_state);
  }

  /**
   * Transforms a map into an array of selected elements.
   *
   * @param array $map
   *   The map.
   *
   * @return array
   *   The resulting list.
   */
  private function cleanupMap(array $map): array {
    $result = [];
    foreach ($map as $item) {
      if ($item) {
        $result[] = $item;
      }
    }
    return $result;
  }

}
